/* 
    6 channel DVM 0-50V range 2017 Adrian Smith. 
    Version 3 - Written for 20x4 character LCD with temperature display on top line
    
    ***Calibration***
    
    Use 1% tolerance resistors for the voltage divider and calibrate as below:-
    
    VREF= 5V supply voltage (measured with precision DVM to get best accuracy)
    
    The voltage divider factor is calculated by dividing the first voltage by the second voltage or:
    dividing factor = input voltage ÷ output voltage
    For example, if the first or input voltage measured is 10.02V and the second or output voltage is 0.9V, then the division factor is:
    10.02 ÷ 0.9 = 11.133
    Voltages should be measured with a good calibrated DVM for best accuracy and should be recalibrated once every year.
 */  

#include <LiquidCrystal.h>
#include <OneWire.h>
#include <DallasTemperature.h>

// number of analog samples to take per reading, per channel
#define NUM_SAMPLES 10
// voltage divider calibration values
#define DIV_1    10.9395
#define DIV_2    11.0352
#define DIV_3    11.0382
#define DIV_4    11.2222
#define DIV_5    11.2097
#define DIV_6    11.1602
// ADC reference voltage / calibration value
#define V_REF    readVcc()
#define ONE_WIRE_BUS 10 

// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);
int temp;
int  delayInMillis = 0;
unsigned long lastTempRequest = 0;
int  idle = 0;
unsigned long previousMillis = 0; // last time update
long interval = 10000; // interval at which to do something (milliseconds)

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);
int sum[6] = {0};                // sums of samples taken
unsigned char sample_count = 0;  // current sample number
float voltage[6] = {0.0};        // calculated voltages
char l_cnt = 0;                  // used in 'for' loops

long readVcc() // sets vRef to internal 1.1V reference and calculates Vcc actual voltage
   {
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
    delay(2); // Wait for Vref to settle
    ADCSRA |= _BV(ADSC); // Start conversion
    while (bit_is_set(ADCSRA,ADSC)); // measuring
    uint8_t low = ADCL; // must read ADCL first - it then locks ADCH
    uint8_t high = ADCH; // unlocks both
    long result = (high<<8) | low;
    result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
    return result; // Vcc in millivolts
   }

void setup()
{
    Serial.begin(9600);
    sensors.begin();
    lcd.begin(20, 4);
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("6 Channel DVM");
    lcd.setCursor(0,1);
    lcd.print("2018 Adrian Smith");
    sensors.setResolution(0, 10); // set to 10 bit resolution (0.25 degree increments) 
    sensors.setWaitForConversion(false);
    sensors.requestTemperatures();

delay(2000);


    lcd.clear();
    
}

void sendtoSerial()
{

//send to serial port

Serial.print("A");
Serial.print(voltage[0] * DIV_1, 1); // for diagnostics
Serial.println("");
delay(500);
Serial.print("B");
Serial.print(voltage[1] * DIV_2, 1); // for diagnostics
Serial.println("");
delay(500);
Serial.print("C");
Serial.print(voltage[2] * DIV_3, 1); // for diagnostics
Serial.println("");
delay(500);
Serial.print("D");
Serial.print(voltage[3] * DIV_4, 1); // for diagnostics
Serial.println("");
delay(500);
Serial.print("E");
Serial.print(voltage[4] * DIV_5, 1); // for diagnostics
Serial.println("");
delay(500);
Serial.print("F");
Serial.print(voltage[5] * DIV_6, 1); // for diagnostics
Serial.println("");
delay(500);

   
} 

void loop()
{

// Start temperature sensor readings

    if (millis() - lastTempRequest >= delayInMillis) 
    temp = sensors.getTempCByIndex(0) - 1; // added -1 degree to end to account for inaccuracy of crappy sensor
    sensors.requestTemperatures(); 
    lastTempRequest = millis(); 
        
// End temperature sensor readings
  
    // take a number of analog samples and add them up
    while (sample_count < NUM_SAMPLES) {
        // sample each channel A0 to A5
        for (l_cnt = 0; l_cnt < 6; l_cnt++) {
            sum[l_cnt] += analogRead(A0 + l_cnt);
        }
        sample_count++;
        delay(10);
    }
    // calculate the voltage for each channel
    for (l_cnt = 0; l_cnt < 6; l_cnt++) {
        voltage[l_cnt] = ((float)sum[l_cnt] / (float)NUM_SAMPLES * V_REF) / 1024.0;        
    }
    // display voltages on LCD
    // each voltage is multiplied by the resistor network
    // division factor to calculate the actual voltage

   // lcd.clear(); 
    lcd.setCursor(0,0);
    lcd.print("Load temp. ");
    //lcd.setCursor(12,0);
    lcd.print(temp);
    //lcd.setCursor(15,0);
    lcd.print(" deg C");
    
    // voltage 1 - CH1 24V (pin A0)
    lcd.setCursor(0, 1);
    lcd.print("IP1 ");
    lcd.print(voltage[0] * DIV_1, 1);
    lcd.print("V ");
    
    // voltage 2 - CH2 24V (pin A1)
    lcd.setCursor(10, 1);
    lcd.print("IP2 ");
    lcd.print(voltage[1] * DIV_2, 1);
    lcd.print("V ");
    
    // voltge 3 - CH3 24V (pin A2)
    lcd.setCursor(0, 2);
    lcd.print("IP3 ");
    lcd.print(voltage[2] * DIV_3, 1);
    lcd.print("V ");
    
    // voltage 4 - CH4 24V (pin A3)
    lcd.setCursor(10, 2);
    lcd.print("IP4 ");
    lcd.print(voltage[3] * DIV_4, 1);
    lcd.print("V ");
    
    // voltage 5 - CH5 12V (pin A4)
    lcd.setCursor(0, 3);
    lcd.print("IP5 ");
    lcd.print(voltage[4] * DIV_5, 1);
    lcd.print("V ");
    
    // voltage 6 - CH6 (pin A5)
    lcd.setCursor(10,3);
    lcd.print("IP6 ");
    lcd.print(voltage[5] * DIV_6, 1);
    lcd.print("V ");

     // reset count and sums
    sample_count = 0;
    for (l_cnt = 0; l_cnt < 6; l_cnt++) {
        sum[l_cnt] = 0;
    }

// send data to serial port every 10 seconds

unsigned long currentMillis = millis();

  if(currentMillis - previousMillis > interval) 
  {
     previousMillis = currentMillis;  

    sendtoSerial();
    
  }
  
}
